@@ -1,11 +1,11 @@
-#  File R/InitErgmTerm.multilayer.R in package ergm.multi, part of the
-#  Statnet suite of packages for network analysis, https://statnet.org .
+#  File R/InitErgmTerm.multilayer.R in package ergm.multi, part of the Statnet
+#  suite of packages for network analysis, https://statnet.org .
 #
-#  This software is distributed under the GPL-3 license.  It is free,
-#  open source, and has the attribution requirements (GPL Section 7) at
+#  This software is distributed under the GPL-3 license.  It is free, open
+#  source, and has the attribution requirements (GPL Section 7) at
 #  https://statnet.org/attribution .
 #
-#  Copyright 2003-2024 Statnet Commons
+#  Copyright 2003-2025 Statnet Commons
 ################################################################################
 ## TODO: LL-Constrained proposals.
 ## TODO: Check that noncommutative LL operators work as intended.
@@ -14,6 +14,27 @@
   map(nwl, get.network.attribute, nattr) %>% map(NVL, ~.) %>% map(empty_env) %>% all_identical
 }
 
+.varying_attributes <- function(nwl, lister, getter, type, ignore = c()){
+  attrs1 <- lister(nwl[[1]]) %>% setdiff(ignore)
+  attrs <- nwl[-1] %>% map(lister) %>% map(setdiff, ignore)
+  extra_attrs <- attrs %>% unlist() %>% unique() %>% setdiff(attrs1)
+
+  if(length(extra_attrs))
+    warning(type, " attribute(s) ", paste.and(sQuote(extra_attrs)), " are not found in the first layer and will be ignored.", call.=FALSE, immediate.=TRUE)
+
+  common_attrs <- attrs %>% map(intersect, attrs1)
+  differing <-
+    ! map_lgl(attrs1, function(a)
+      pmap_lgl(
+        list(nwl[-1], common_attrs),
+        function(nw, al) if(a %in% al) identical(getter(nw, a, unlist = FALSE), getter(nwl[[1]], a, unlist = FALSE)) else TRUE
+      ) %>% all())
+  if(any(differing))
+    warning(type, " attribute(s) ", paste.and(sQuote(attrs1[differing])), " have values different from those in the first layer and will be overwritten.", call.=FALSE, immediate.=TRUE)
+
+  length(extra_attrs) || any(differing)
+}
+
 .layer_namemap <- function(nw){
   nwnames <- .peek_vattrv(nw, ".LayerName", missing="NULL")
   if(is.numeric(nwnames)) nwnames <- NULL
@@ -40,6 +61,10 @@
   if(collapse) ergm_mk_std_op_namewrap("L", reprs) else reprs
 }
 
+assert_LHS_Layer <- function(nw, errfn = ergm_Init_abort){
+  if(anyNA(.peek_vattrv(nw, ".LayerID"))) errfn("The LHS of the model is not a multilayer ", sQuote("Layer()"), " construct.")
+}
+
 #' Construct a "view" of a network.
 #'
 #' Returns a network with edges optionally filtered according to a
@@ -192,7 +217,9 @@ direct.network <- function(x, rule=c("both", "upper", "lower")){
                upper = cbind(pmin(el[,1],el[,2]),pmax(el[,1],el[,2])),
                lower = cbind(pmax(el[,1],el[,2]),pmin(el[,1],el[,2])))
   
-  o <- network.initialize(network.size(x), directed=TRUE, bipartite=x%n%"bipartite", loops=has.loops(x), hyper=is.hyper(x), multiple=is.multiplex(x))
+  o <- network.initialize(network.size(x), directed = TRUE,
+                          bipartite = b1.size(x), loops = has.loops(x),
+                          hyper = is.hyper(x), multiple = is.multiplex(x))
   o <- network.edgelist(el, o)
   nvattr.copy.network(o, x)
 }
@@ -224,13 +251,14 @@ direct.network <- function(x, rule=c("both", "upper", "lower")){
 #' @param .bipartite If the layer specification is via a single
 #'   network with edge attributes and the network is unipartite, an
 #'   optional integer vector to specify which of the layers should be
-#'   treated as bipartite and how many `b1` vertices there are.
-#' @param .active A [nodal attribute specification][nodal_attributes]
-#'   (`? nodal_attributes`) specifying which nodes on each network
-#'   *may* have ties, or a list with an element for each network. The
-#'   list will be recycled up to the number of layers.
+#'   treated as bipartite and how many mode 1 vertices there are.
+#' @param .active An optional list with a [nodal attribute
+#'   specification][nodal_attributes] (`? nodal_attributes`) for each
+#'   layer, specifying which nodes on each layer *may* have ties.
 #'
-#' @return A network object with layer metadata.
+#' @return A [`network`] object comprising the provided layers, with layer metadata.
+#' @templateVar combiner Layer
+#' @template combine_networks_readonly
 #'
 #' @note The resulting network will be the "least common denominator"
 #'   network: if not all layers have the same bipartedness, all layers
@@ -240,7 +268,12 @@ direct.network <- function(x, rule=c("both", "upper", "lower")){
 #'   used to construct a bipartite subgraph of a unipartite graph or
 #'   change directedness.
 #'
-#' @section Specifying models for multilayer network:
+#'   Its nonstandard network and vertex attributes will be taken from
+#'   the *first* network in the list. The subsequent networks'
+#'   attributes will be overwritten with a warning if they differ from
+#'   those in the first network.
+#'
+#' @section Specifying models for multilayer networks:
 #' In order to fit a model for multilayer
 #' networks, first use [`Layer`] construct an LHS network that
 #' [ergm()] will understand as multilayered.
@@ -293,7 +326,7 @@ direct.network <- function(x, rule=c("both", "upper", "lower")){
 #' \dQuote{logical} layer that has ties that are in \code{nwA} but not in
 #' \code{nwB}.
 #'
-#' Transpose function \code{\link{t}} applied to a directed layer will reverse
+#' Transpose function [t()] applied to a directed layer will reverse
 #' the direction of all relations (transposing the sociomatrix). Unlike the
 #' others, it can only be used on an observed layer directly. For example,
 #' \code{~t(`1`)&t(`2`)} is valid but \code{~t(`1`&`2`)} is not.
@@ -435,18 +468,23 @@ Layer <- function(..., .symmetric=NULL, .bipartite=NULL, .active=NULL){
 
   }else stop("Unrecognized format for multilayer specification. See help for information.")
 
+  ## If network or vertex attributes differ from the first network, warn.
+  .varying_attributes(nwl, list.network.attributes, get.network.attribute, "Network", ignore = c("directed", "bipartite", "mnext", ".block_blacklist"))
+  .varying_attributes(nwl, list.vertex.attributes, get.vertex.attribute, "Vertex", ignore = c(".undirected", ".bipartite", ".ubid"))
+
+  for(i in seq_along(nwl)[-1L])
+    nwl[[i]] <- nvattr.copy.network(nwl[[i]], nwl[[1]], ignore = c(eval(formals(nvattr.copy.network)$ignore), ".block_blacklist", ".ubid", ".undirected", ".bipartite"))
+
   if(!is.null(.active)){
-    if(!is.list(.active)) .active <- list(.active)
-    .active <- rep(.active, length.out=length(nwl))
-    al <- Map(function(nw, a) ergm_get_vattr(a, nw, accept="logical"),
-              nwl, .active)
+    if(!is.list(.active) || length(.active) != length(nwl)) stop(sQuote(".active="), " argument if given must be a list of attribute specifications, one for each layer.")
+    al <- map(.active, ergm_get_vattr, nwl[[1]], accept="logical")
     if(!all(unlist(al)))
       nwl <- Map(function(nw, a) blacklist_intersect(nw, a, invert=TRUE),
                  nwl, al)
   }
 
   # nwl may now be a list with networks of heterogeneous bipartitedness.
-  bip <- sapply(nwl, `%n%`, "bipartite") %>% sapply(NVL, 0L)
+  bip <- sapply(nwl, b1.size) %>% sapply(NVL, 0L)
   blockout <- if(all_identical(bip)) rep(FALSE, length(nwl)) else bip
 
   nwl <- Map(function(nw, b){
@@ -491,7 +529,7 @@ Layer <- function(..., .symmetric=NULL, .bipartite=NULL, .active=NULL){
 
   if(!.same_constraints(nwl, "constraints")) stop("Layers have differing constraint structures. This is not supported at this time.")
   if(!.same_constraints(nwl, "obs.constraints")) stop("Layers have differing observation processes. This is not supported at this time.")
-  
+
   nw <- combine_networks(nwl, blockID.vattr=".LayerID", blockName.vattr=".LayerName", ignore.nattr = c(eval(formals(combine_networks)$ignore.nattr), "constraints", "obs.constraints", "ergm"), subnet.cache=TRUE)
 
   nw %n% "ergm" <- combine_ergmlhs(nwl)
@@ -521,10 +559,16 @@ Layer <- function(..., .symmetric=NULL, .bipartite=NULL, .active=NULL){
 ## }
 
 .depends_on_layers <- function(commands){
-  coms <- commands[-1]
-  if(any(coms==0)) coms <- coms[-(which(coms==0)+1)] # Drop all numeric literals (i.e., numbers preceded by 0).
-  coms <- coms[coms>=1] # Drop all commands.
-  unique(coms)
+  ## Drop all numeric literals and filter references.
+  i <- 1
+  while(i <= length(commands)){
+    while(commands[i]==0)
+      commands <- commands[c(-i,-i-1)]
+    i <- i + 1
+  }
+
+  commands <- commands[commands>=1] # Drop all commands.
+  unique(commands)
 }
 
 InitErgmTerm..layer.net <- function(nw, arglist, ...){
@@ -534,7 +578,8 @@ InitErgmTerm..layer.net <- function(nw, arglist, ...){
                       defaultvalues = list(NULL),
                       required = c(TRUE))
 
-  
+  assert_LHS_Layer(nw)
+
   nwl <- subnetwork_templates(nw,".LayerID",".LayerName")
 
   ll <- to_ergm_Cdouble(.set_layer_namemap(a$L, nw))
@@ -582,6 +627,8 @@ LL_POSTOPMAP <- list(
       `round` = -19)
 )
 
+LL_STOP <- -.Machine$integer.max
+
 LL_IDEMPOTENT <- c("&", "&&", "|", "||")
 LL_TAUTOLOGICAL <- c("==", ">=", "<=")
 LL_CONTRADICTORY <- c("!=", "xor", "<", ">")
@@ -640,13 +687,13 @@ sub.ergm_LayerLogic <- function(x){
 
   simplify <- function(call){
     if(is.call(call)){
-      op <- call[[1]]
+      op <- as.character(call[[1]])
 
       call[-1] <- lapply(call[-1], simplify)
 
-      if(as.character(op) %in% LL_IDEMPOTENT && all_identical(as.list(call)[-1])) call[[2]]
-      else if(as.character(op) %in% LL_TAUTOLOGICAL && all_identical(as.list(call)[-1])) TRUE
-      else if(as.character(op) %in% LL_CONTRADICTORY && length(call)%%2==0 && all_identical(as.list(call)[-1])) FALSE
+      if(op %in% LL_IDEMPOTENT && all_identical(as.list(call)[-1])) call[[2]]
+      else if(op %in% LL_TAUTOLOGICAL && all_identical(as.list(call)[-1])) TRUE
+      else if(op %in% LL_CONTRADICTORY && length(call)%%2==0 && all_identical(as.list(call)[-1])) FALSE
       else call
     }else{
       lidSub(call)
@@ -673,15 +720,16 @@ to_ergm_Cdouble.ergm_LayerLogic <- function(x, ...){
 
   postfix <- function(call, coml=c()){
     if(is.call(call)){
-      op <- call[[1]]
-      if(as.character(op) %in% unlist(lapply(LL_PREOPMAP, names))){
-        coml <- c(coml, LL_PREOPMAP[[length(call)-1]][[as.character(op)]])
+      op <- as.character(call[[1]])
+      if(op %in% unlist(lapply(LL_PREOPMAP, names))){
+        coml <- c(coml, LL_PREOPMAP[[length(call)-1]][[op]])
         postop <- FALSE
       }else postop <- TRUE
-      for(i in seq_along(call[-1])+1){
+      ## Push in reverse order, so they pop in the right order.
+      for(i in rev(seq_along(call[-1])+1)){
         coml <- c(coml, postfix(call[[i]]))
       }
-      if(postop) coml <- c(coml, LL_POSTOPMAP[[length(call)-1]][as.character(op)])
+      if(postop) coml <- c(coml, LL_POSTOPMAP[[length(call)-1]][op])
     }else{
       coml <- c(coml, lidMap(call))
     }
@@ -689,15 +737,17 @@ to_ergm_Cdouble.ergm_LayerLogic <- function(x, ...){
   }
   
   com <- postfix(ult(formula))
-  c(sum(com!=0 & !com%in%unlist(LL_PREOPMAP)), com)
+  c(com, LL_STOP)
 }
 
 test_eval.LayerLogic <- function(commands, lv, lvr = lv){
-  coms <- as.integer(commands[-1])
-  lv <- rep(lv, length.out=max(coms))
+  coms <- as.integer(commands)
+  ndeps <- EVL3(.depends_on_layers(commands), max(.), 0)
+  lv <- rep_len(lv, max(ndeps))
+  lvr <- rep_len(lvr, max(ndeps))
   stack <- integer(0)
-  if(sum(coms!=0 & !coms%in%unlist(LL_PREOPMAP))!=commands[1]) stop("Layer specification command vector specifies incorrect number of commands.", call.=FALSE)
-  for(i in 1:commands[1]){
+
+  repeat{
     com <- coms[1]
     if(com==0){
       coms <- coms[-1]
@@ -783,6 +833,8 @@ test_eval.LayerLogic <- function(commands, lv, lvr = lv){
       coms <- coms[-1]
       x0 <- coms[1]
       stack <- c(lvr[x0], stack)
+    }else if(com==LL_STOP){
+      break
     }else{
       stack <- c(lv[com], stack)
     }
@@ -842,6 +894,8 @@ InitErgmTerm.L <- function(nw, arglist, ...){
                       defaultvalues = list(NULL, empty_env(~.)),
                       required = c(TRUE, FALSE))
 
+  assert_LHS_Layer(nw)
+
   nwl <- subnetwork_templates(nw,".LayerID",".LayerName")
 
   Ls <- .set_layer_namemap(a$Ls, nw)
@@ -862,7 +916,7 @@ InitErgmTerm.L <- function(nw, arglist, ...){
   wm <- wrap.ergm_model(m, nw1, function(x) .lspec_coef.namewrap(list(a$Ls))(x))
   gs <- wm$emptynwstats
   wm$emptynwstats <- if(!is.null(gs)) gs*nltrms
-  wm$dependence <- wm$dependence || !is.dyad.independent(auxiliaries, basis=nw, ignore_aux=FALSE)
+  wm$dependence <- wm$dependence || NA # If not determined by the model, set based on the layer logic.
 
   c(list(name="OnLayer", iinputs=nltrms, inputs=w, submodel=m, auxiliaries = auxiliaries),
     wm)
@@ -900,6 +954,8 @@ InitErgmTerm.CMBL <- function(nw, arglist, ...){
                       defaultvalues = list(empty_env(~.)),
                       required = c(FALSE))
 
+  assert_LHS_Layer(nw)
+
   Ls <- .set_layer_namemap(a$Ls, nw)
   auxiliaries <- .mk_.layer.net_auxform(Ls)
   nltrms <- length(list_rhs.formula(auxiliaries))
@@ -920,25 +976,27 @@ InitErgmTerm.CMBL <- function(nw, arglist, ...){
 #' @templateVar Ls.howmany two
 #' @templateVar Ls.interp {} specifying the layers of interest.
 #' @template ergmTerm-Ls
-#' @param type determines what is counted:
-#'   1) *"any"* Number of configurations
-#'   \eqn{(i-j), (i-k)}{(i,j), (i,k)} , where
-#'   \eqn{(i-j)}{(i,j)} is in logical layer `Ls[[1]]`
-#'   and \eqn{(i-k)}{(i,k)} is in logical layer `Ls[[2]]` .
-#'   2) *"out"* Number of configurations
+#' @param type if the network is directed, an argument to determine which configurations are counted:
+#'   \describe{
+## #'   \item{`"any"`}{Number of configurations
+## #'   \eqn{(i-j), (i-k)}{(i,j), (i,k)}, where
+## #'   \eqn{(i-j)}{(i,j)} is in logical layer `Ls[[1]]`
+## #'   and \eqn{(i-k)}{(i,k)} is in logical layer `Ls[[2]]`.}
+#'   \item{`"out"`}{Number of configurations
 #'   \eqn{(i{\rightarrow}j), (i{\rightarrow}k)}{(i,j), (i,k)}, where
 #'   \eqn{(i{\rightarrow}j)}{(i,j)} is in logical layer `Ls[[1]]`
-#'   and \eqn{(i{\rightarrow}k)}{(i,k)} is in logical layer `Ls[[2]]`.
-#'   3) *"in"* Number of configurations
+#'   and \eqn{(i{\rightarrow}k)}{(i,k)} is in logical layer `Ls[[2]]`.}
+#'   \item{`"in"`}{Number of configurations
 #'   \eqn{(j{\rightarrow}i), (k{\rightarrow}i)}{(j,i), (k,i)}, where
 #'   \eqn{(j{\rightarrow}i)}{(j,i)} is in logical layer `Ls[[1]]`
-#'   and \eqn{(k{\rightarrow}i)}{(k,i)} is in logical layer `Ls[[2]]`.
-#'   4) *"path"* Number of configurations
+#'   and \eqn{(k{\rightarrow}i)}{(k,i)} is in logical layer `Ls[[2]]`.}
+#'   \item{`"path"`}{Number of configurations
 #'   \eqn{(j{\rightarrow}i), (i{\rightarrow}k)}{(j,i), (i,k)}, where
 #'   \eqn{(j{\rightarrow}i)}{(j,i)} is in logical layer `Ls[[1]]`
-#'   and \eqn{(i{\rightarrow}k)}{(i,k)} is in logical layer `Ls[[2]]`.
+#'   and \eqn{(i{\rightarrow}k)}{(i,k)} is in logical layer `Ls[[2]]`.}
+#'   }
 #'
-#'   At this time, `"any"` is only supported for undirected networks, and if the network is undirected, `type` is ignored and `"any"` is assumed.
+#'   This argument is ignored for undirected networks.
 #'
 #' @param distinct if `TRUE`, \eqn{j} and \eqn{k} above are required to
 #'   be distinct. That is, the constituent edges may not be coincident or
@@ -954,12 +1012,19 @@ InitErgmTerm.twostarL<-function(nw, arglist,  ...) {
                       varnames = c("Ls", "type", "distinct"),
                       vartypes = c("formula,list", "character", "logical"),
                       defaultvalues = list(NULL, NULL, TRUE),
-                      required = c(TRUE, TRUE, FALSE))
+                      required = c(TRUE, FALSE, FALSE))
+
+  assert_LHS_Layer(nw)
+
   TYPES <- c("any", "out", "in", "path")
   TYPEREP <- setNames(c("--", "<>", "><", ">>"), TYPES)
-  type <- match.arg(tolower(a$type), TYPES)
-  if(!is.directed(nw)) type <- "any"
-  else if(type == "any") ergm_Init_abort(paste0("at this time, ", sQuote('type="any"'), " is only supported for undirected networks"))
+
+  type <- a$type
+  if (!is.directed(nw)) type <- "any"
+  else if (is.null(type)) ergm_Init_stop(sQuote("type"), " argument is required for directed networks")
+
+  type <- match.arg(tolower(type), TYPES)
+  if (type == "any" && is.directed(nw)) ergm_Init_abort(paste0("at this time, ", sQuote('type="any"'), " is only supported for undirected networks"))
   typeID <- match(type, TYPES) - 1L
 
   Ls <- .set_layer_namemap(a$Ls, nw)
@@ -976,3 +1041,120 @@ InitErgmTerm.twostarL<-function(nw, arglist,  ...) {
   iinputs <- c(typeID, a$distinct)
   list(name="twostarL", coef.names=coef.names, iinputs=iinputs, auxiliaries=auxiliaries, minval=0, dependence=TRUE)
 }
+
+################################################################################
+
+#' @templateVar name mutualL
+#' @title Mutuality
+#' @description In binary ERGMs, equal to the number of
+#'   pairs of actors \eqn{i} and \eqn{j} for which \eqn{(i{\rightarrow}j)}{(i,j)}
+#'   and \eqn{(j{\rightarrow}i)}{(j,i)} both exist.
+#'
+#' @details This term can only be used with directed networks.
+#'
+#' @usage
+#' # binary: mutualL(same=NULL, diff=FALSE, by=NULL, keep=NULL, Ls=NULL)
+#' @param same optional argument. If passed the name of a vertex attribute,
+#'   only mutual pairs that match on the attribute are counted. Only one of `same`
+#'   or `by` may be used. If both parameters are passed, `same` takes precedent. This
+#'   parameter is affected by `diff`.
+#' @param diff separate counts for each unique matching value can be obtained by using
+#'   `diff=TRUE` with `same`.
+#' @param by each node is counted separately for each mutual pair in which it
+#'   occurs and the counts are tabulated by unique values of the attribute if
+#'   passed the name of a vertex attribute. This means that the sum of the mutual statistics when `by` is used
+#'   will equal twice the standard mutual statistic. Only one of `same`
+#'   or `by` may be used. If both parameters are passed, `same` takes precedent. This
+#'   parameter is not affected by `diff`.
+#' @param keep a numerical vector to specify which statistics should be kept whenever the `mutual` term would
+#'   ordinarily result in multiple statistics.
+#' @templateVar Ls.howmany one or two
+#' @templateVar Ls.interp . If given, the statistic will count the number of dyads where a tie in `Ls[[1]]` reciprocates a tie in `Ls[[2]]` and vice versa. (Note that dyad that has mutual ties in `Ls[[1]]` and in `Ls[[2]]` will add 2 to this statistic.) If a formula is given, it is replicated.
+#' @template ergmTerm-Ls
+#'
+#' @template ergmTerm-general
+#'
+#' @concept directed
+#' @concept frequently-used
+#' @concept layer-aware
+InitErgmTerm.mutualL<-function (nw, arglist, ...) {
+  ## Check the network and arguments to make sure they are appropriate.
+  a <- check.ErgmTerm(nw, arglist, directed=TRUE, bipartite=NULL,
+                      varnames = c("same", "by", "diff", "keep", "Ls"),
+                      vartypes = c("character", "character", "logical", "numeric", "formula,list"),
+                      defaultvalues = list(NULL, NULL, FALSE, NULL, NULL),
+                      required = c(FALSE, FALSE, FALSE, FALSE, FALSE))
+
+  assert_LHS_Layer(nw)
+
+  ## Process the arguments
+  if (!is.null(a$same) || !is.null(a$by)) {
+    if (!is.null(a$same)) {
+     attrname <- a$same
+     if (!is.null(a$by)) 
+       warning("Ignoring 'by' argument to mutual because 'same' exists", call.=FALSE)
+    }else{
+     attrname <- a$by
+    }
+    nodecov <- get.node.attr(nw, attrname)
+    u <- NVL(a$levels, sort(unique(nodecov)))
+    if (!is.null(a$keep)) {
+      u <- u[a$keep]
+    }
+    #   Recode to numeric
+    nodecov <- match(nodecov,u,nomatch=length(u)+1)
+    # All of the "nomatch" should be given unique IDs so they never match:
+    dontmatch <- nodecov==(length(u)+1)
+    nodecov[dontmatch] <- length(u) + (1:sum(dontmatch))
+    ui <- seq(along=u)
+  }
+
+  ### Construct the list to return
+  if (!is.null(a$same) || !is.null(a$by)) {
+    if (is.null(a$same)) {
+      coef.names <- paste("mutual.by", attrname, u, sep=".")
+      inputs <- c(ui, nodecov)
+    }else{
+     if (a$diff) {
+      coef.names <- paste("mutual.same", attrname, u, sep=".")
+      inputs <- c(ui, nodecov)
+     }else{ 
+      coef.names <- paste("mutual", attrname, sep=".")
+      inputs <- nodecov
+     }
+    }
+    if (is.null(a$same) && !is.null(a$by)) {
+     name <- "mutual_by_attr"
+    }else{
+     name <- "mutual"
+    }
+  }else{
+     name <- "mutual"
+     coef.names <- "mutual"
+     inputs <- NULL
+  }
+
+  maxval <- network.dyadcount(nw,FALSE)/2
+
+  Ls <- .set_layer_namemap(a$Ls, nw)
+  if(is(Ls,"formula")) Ls <- list(Ls)
+  L1 <- Ls[[1]]
+  L2 <- Ls[[2]]
+  if(!is.null(L1) || !is.null(L2)){
+    NVL(L1) <- L2
+    NVL(L2) <- L1
+    auxiliaries <- .mk_.layer.net_auxform(L1)
+    aux2 <- .mk_.layer.net_auxform(L2)
+    auxiliaries[[2]] <- call("+", auxiliaries[[2]], aux2[[2]])
+    name <- paste(name, "ML", sep="_")
+    coef.names <- .lspec_coef.namewrap(if(L1==L2) list(L1) else list(L1,L2))(coef.names)
+    maxval <- maxval*2
+  }else auxiliaries <- NULL
+  
+  list(name=name,                      #name: required
+       coef.names = coef.names,        #coef.names: required
+       inputs=inputs,
+       auxiliaries = auxiliaries,
+       minval = 0,
+       maxval = maxval) 
+}
