@@ -1,11 +1,11 @@
-#  File R/InitErgmTerm.multinet.R in package ergm.multi, part of the
-#  Statnet suite of packages for network analysis, https://statnet.org .
+#  File R/InitErgmTerm.multinet.R in package ergm.multi, part of the Statnet
+#  suite of packages for network analysis, https://statnet.org .
 #
-#  This software is distributed under the GPL-3 license.  It is free,
-#  open source, and has the attribution requirements (GPL Section 7) at
+#  This software is distributed under the GPL-3 license.  It is free, open
+#  source, and has the attribution requirements (GPL Section 7) at
 #  https://statnet.org/attribution .
 #
-#  Copyright 2003-2024 Statnet Commons
+#  Copyright 2003-2025 Statnet Commons
 ################################################################################
 #' A multinetwork network representation.
 #'
@@ -19,10 +19,13 @@
 #'
 #'   1. Several networks as (optionally named) arguments.
 #'
-#' @return A network object with multinetwork metadata.
+#' @return A [`network`] object comprising the provided networks, with multinetwork metadata.
 #'
-#' @seealso [Help on model specification][ergmTerm] for specific terms
-#' @seealso `vignette("Goeyvaerts_reproduction")` for a demonstration
+#' @templateVar combiner Networks
+#' @template combine_networks_readonly
+#'
+#' @seealso [`ergmTerm`] for specific terms.
+#' @seealso `vignette("Goeyvaerts_reproduction")` for a demonstration.
 #' @examples
 #'
 #' data(samplk)
@@ -82,11 +85,12 @@ InitErgmTerm..subnets <- function(nw, arglist, ...){
 #' @param attrnames a list (or a selection index) for attributes to obtain; for combined networks, defaults to all.
 #' @param unit whether to obtain edge, vertex, or network attributes.
 #' @template ergmTerm-NetworkIDName
+#' @param store.nid whether to include columns with network ID and network name; the columns will be named with the arguments passed to `.NetworkID` and `.NetworkName`.
 #' @param ... additional arguments, currently passed to unlist()].
 #'
 #' @seealso [network::as_tibble.network()]
 #' @export
-as_tibble.combined_networks<-function(x,attrnames=(match.arg(unit)%in%c("vertices","networks")), ..., unit=c("edges", "vertices", "networks"), .NetworkID=".NetworkID", .NetworkName=".NetworkName"){
+as_tibble.combined_networks<-function(x,attrnames=(match.arg(unit)%in%c("vertices","networks")), ..., unit=c("edges", "vertices", "networks"), .NetworkID=".NetworkID", .NetworkName=".NetworkName", store.nid=FALSE){
   unit <- match.arg(unit)
   if(unit!="networks") return(NextMethod())
 
@@ -102,7 +106,14 @@ as_tibble.combined_networks<-function(x,attrnames=(match.arg(unit)%in%c("vertice
   if(is.logical(attrnames) || is.numeric(attrnames)) attrnames <- na.omit(names(al)[attrnames])
   else intersect(attrnames, names(al))
 
-  al[attrnames] %>% lapply(simplify_simple, ...) %>% as_tibble()
+  out <- al[attrnames] %>% lapply(simplify_simple, ...) %>% as_tibble()
+
+  if(store.nid){
+    out[[.NetworkID]] <- unique(x %v% .NetworkID)
+    out[[.NetworkName]] <- unique(x %v% .NetworkName)
+  }
+
+  out
 }
 
 get_lminfo <- function(nattrs, lm=~1, subset=TRUE, contrasts=NULL, offset=NULL, weights=1){
@@ -141,6 +152,14 @@ get_lminfo <- function(nattrs, lm=~1, subset=TRUE, contrasts=NULL, offset=NULL,
   list(xf=xf, xm=xm, subset=subset, offset=offset)  
 }
 
+assert_LHS_Networks <- function(nw, nid, term_trace = TRUE, call = if(term_trace) NULL else rlang::caller_env()){
+  if(anyNA(.peek_vattrv(nw, nid))){
+    msg <- paste0("The LHS of the model is not a multi-network ", sQuote("Networks()"), " construct.")
+    if(term_trace) ergm_Init_abort(msg, call=call)
+    else abort(msg, call=call)
+  }
+}
+
 #' @import purrr
 #' @import tibble
 #' @templateVar name N
@@ -158,6 +177,10 @@ get_lminfo <- function(nattrs, lm=~1, subset=TRUE, contrasts=NULL, offset=NULL,
 #' error. This can be avoided by pre-filtering with `subset`, which
 #' controls which networks are affected by the term.
 #'
+#' The formula may also reference `.NetworkID` and `.NetworkName`. In
+#' particular, `~0+factor(.NetworkID)` will evaluate `formula` on each
+#' network individually.
+#'
 #' @note Care should be taken to avoid multicollinearity when using
 #'   this operator. As with the [lm()] function, `lm` formulas have an
 #'   implicit intercept, which can be suppressed by specifying `~ 0 +
@@ -239,12 +262,14 @@ InitErgmTerm.N <- function(nw, arglist, ..., N.compact_stats=TRUE, .NetworkID=".
                       defaultvalues = list(NULL, ~1, TRUE, 1, NULL, NULL, NULL, .NetworkID, .NetworkName),
                       required = c(TRUE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE))
 
+  assert_LHS_Networks(nw, a$.NetworkID)
+
   auxiliaries <- eval(substitute(~.subnets(.NetworkID), list(.NetworkID=a$.NetworkID)), baseenv())
 
   nwl <- subnetwork_templates(nw, a$.NetworkID, a$.NetworkName)
   nwnames <- names(nwl)
   nn <- length(nwl)
-  nattrs <- as_tibble(nw, unit="networks", .NetworkID=a$.NetworkID, .NetworkName=a$.NetworkName)
+  nattrs <- as_tibble(nw, unit="networks", .NetworkID=a$.NetworkID, .NetworkName=a$.NetworkName, store.nid=TRUE)
 
   lmi <- get_lminfo(nattrs, lm=a$lm, subset=a$subset, contrasts=a$contrasts, offset=a$offset, weights=a$weights)
 
@@ -435,6 +460,8 @@ InitErgmTerm.ByNetDStats <- function(nw, arglist, ..., .NetworkID=".NetworkID"){
                       defaultvalues = list(NULL, TRUE, .NetworkID),
                       required = c(TRUE, FALSE, FALSE))
 
+  assert_LHS_Networks(nw, a$.NetworkID)
+
   auxiliaries <- eval(substitute(~.subnets(.NetworkID), list(.NetworkID=a$.NetworkID)), baseenv())
   nattrs <- as_tibble(nw, unit="networks")
 
