@@ -1,11 +1,11 @@
-#  File R/combine.networks.R in package ergm.multi, part of the
-#  Statnet suite of packages for network analysis, https://statnet.org .
+#  File R/combine.networks.R in package ergm.multi, part of the Statnet suite
+#  of packages for network analysis, https://statnet.org .
 #
-#  This software is distributed under the GPL-3 license.  It is free,
-#  open source, and has the attribution requirements (GPL Section 7) at
+#  This software is distributed under the GPL-3 license.  It is free, open
+#  source, and has the attribution requirements (GPL Section 7) at
 #  https://statnet.org/attribution .
 #
-#  Copyright 2003-2024 Statnet Commons
+#  Copyright 2003-2025 Statnet Commons
 ################################################################################
 
 #' A single block-diagonal network created by combining multiple networks
@@ -134,8 +134,8 @@ combine_networks <- function(nwl, ignore.nattr=c("mnext"), ignore.vattr=c(), ign
   ns <- sapply(nwl, network.size)
   blks <- c(0, cumsum(ns))
 
-  out <- network.initialize(sum(ns), directed=is.directed(nwl[[1]]))
-
+  constructor <- if(is(nwl[[1]], "networkLite")) networkLite::networkLite else network.initialize
+  out <-  constructor(sum(ns), directed=is.directed(nwl[[1]]))
 
   # Concatenate network attributes. If you run into what looks like a covariate matrix, combine it correctly.
   sna <- list()
@@ -190,15 +190,14 @@ combine_networks <- function(nwl, ignore.nattr=c("mnext"), ignore.vattr=c(), ign
 
   # Add ties and attributes
 
-  for(b in seq_along(nwl)){
-    el <- rbind(as.edgelist(nwl[[b]]),as.edgelist(is.na(nwl[[b]])))
-    eids <- apply(el, 1, function(e) get.edgeIDs(nwl[[b]], e[1], e[2], na.omit=FALSE))
+  el <- map(seq_along(nwl), function(b) {
+    df <- as_tibble(nwl[[b]], attrname = TRUE, unit = "edge", na.rm = FALSE)
+    names <- rep(list(as.list(names(df)[-(1:2)])), nrow(df))
+    vals <- transpose(df[,-(1:2)])
+    list(tails = df[[1]]+blks[b], heads = df[[2]]+blks[b], names = names, vals = vals)
+  }) %>% transpose()
 
-    vals <- lapply(nwl[[b]]$mel,"[[","atl")[eids]
-    names <- lapply(vals, names)
-
-    out <- add.edges(out, el[,1]+blks[b], el[,2]+blks[b], names.eval=names, vals.eval=vals)
-  }
+  out <- add.edges(out, unlist(el$tails), unlist(el$heads), names.eval=do.call(c, el$names), vals.eval=do.call(c, el$vals))
 
   # Finally, add a vertex attribute specifying the blocks
 
@@ -235,12 +234,13 @@ combine_networks <- function(nwl, ignore.nattr=c("mnext"), ignore.vattr=c(), ign
   attrset <- if(keep.unshared.attr) union else intersect
 
   ns <- sapply(nwl, network.size)
-  es <- sapply(nwl, "%n%", "bipartite")
+  es <- sapply(nwl, b1.size)
   eblks <- c(0, cumsum(es))
   bip <- eblks[length(eblks)]
   ablks <- cumsum(c(bip, ns-es))
 
-  out <- network.initialize(sum(ns), directed=is.directed(nwl[[1]]), bipartite=bip)
+  constructor <- if(is(nwl[[1]], "networkLite")) networkLite::networkLite else network.initialize
+  out <-  constructor(sum(ns), directed=is.directed(nwl[[1]]), bipartite=bip)
 
   # Concatenate network attributes. If you run into what looks like a covariate matrix, combine it correctly.
   sna <- list()
@@ -299,15 +299,14 @@ combine_networks <- function(nwl, ignore.nattr=c("mnext"), ignore.vattr=c(), ign
 
   # Add ties and attributes
 
-  for(b in seq_along(nwl)){
-    el <- rbind(as.edgelist(nwl[[b]]),as.edgelist(is.na(nwl[[b]])))
-    eids <- apply(el, 1, function(e) get.edgeIDs(nwl[[b]], e[1], e[2], na.omit=FALSE))
+  el <- map(seq_along(nwl), function(b) {
+    df <- as_tibble(nwl[[b]], attrname = TRUE, unit = "edge", na.rm = FALSE)
+    names <- rep(list(as.list(names(df)[-(1:2)])), nrow(df))
+    vals <- transpose(df[,-(1:2)])
+    list(tails = df[[1]]+eblks[b], heads = df[[2]]-es[b]+ablks[b], names = names, vals = vals)
+  }) %>% transpose()
 
-    vals <- lapply(nwl[[b]]$mel,"[[","atl")[eids]
-    names <- lapply(vals, names)
-
-    out <- add.edges(out, el[,1]+eblks[b], el[,2]-es[b]+ablks[b], names.eval=names, vals.eval=vals)
-  }
+  out <- add.edges(out, unlist(el$tails), unlist(el$heads), names.eval=do.call(c, el$names), vals.eval=do.call(c, el$vals))
 
   # Finally, add a vertex attribute specifying the blocks
 
@@ -436,6 +435,8 @@ split.network <- function(x, f, drop = FALSE, sep = ".", lex.order = FALSE, ...)
 #'
 #' @export
 uncombine_network <- function(nw, split.vattr=nw %n% ".blockID.vattr", names.vattr=nw %n% ".blockName.vattr", use.subnet.cache=FALSE){
+  if(!is(nw, "combined_networks")) stop("Specified network was not constructed by ", sQuote("combine_networks()"), ".")
+
   tmp <- .pop_vattrv(nw, split.vattr); nw <- tmp$nw; f <- tmp$vattr
   if(!is.null(names.vattr)){ tmp <- .pop_vattrv(nw, names.vattr); nw <- tmp$nw; nwnames <- tmp$vattr }
 
@@ -460,6 +461,29 @@ uncombine_network <- function(nw, split.vattr=nw %n% ".blockID.vattr", names.vat
   nwl
 }
 
+#' Dynamic registry of functions that combine networks.
+#'
+#' This is used primarily by developers to provide informative error
+#' messages.
+#'
+#' @param blockID the vertex attribute used as the ID of the block.
+#'
+#' @param combiner a character vector of length 1 or 2, giving the
+#'   top-level (i.e. user-visible) function that combines on it and
+#'   optionally what is being combined.
+#'
+#' @keywords internal
+#' @export
+ergm.multi_combiner <- local({
+  cache <- list()
+
+  function(blockID, combiner=NULL){
+    if(missing(blockID)) cache
+    else if(is.null(combiner)) cache[[blockID]]
+    else cache[[blockID]] <<- c(combiner, tolower(combiner))
+  }
+})
+
 #' Obtain empty networks representing constituents of a combined network
 #'
 #' This utility uncombines a [combine_networks()] network using subnetwork cache (which contains only empty networks). It is used primarily by initialisation functions.
@@ -472,7 +496,20 @@ uncombine_network <- function(nw, split.vattr=nw %n% ".blockID.vattr", names.vat
 #'
 #' @keywords internal
 #' @export
-subnetwork_templates <- function(nw, split.vattr=".NetworkID", names.vattr=".NetworkName", copy.ergmlhs=c("response")){
+subnetwork_templates <- function(nw, split.vattr=nw%n%".blockID.vattr", names.vattr=nw%n%".blockName.vattr", copy.ergmlhs=c("response")){
+  if(NVL3(nw.split.vattr<-nw%n%".blockID.vattr", split.vattr != ., FALSE)){
+    ergm_Init_stop(
+      "The LHS was (at the top level) created by ",
+      NVL3(ergm.multi_combiner(nw.split.vattr)[1],
+           sQuote(.),
+           paste0("an unknown function that uses attribute ", sQuote(nw.split.vattr))),
+      " but the term is trying to extract its ",
+      NVL3(ergm.multi_combiner(split.vattr)[2],
+           paste0(., " (created by ", sQuote(ergm.multi_combiner(split.vattr)[1]), ")."),
+           paste0("subgraphs defined by ", split.vattr)),
+      " Nesting of terms must match the nesting of constructors; this may change in the future.", immediate.=TRUE)
+  }
+
   uncombine_network(nw, split.vattr=split.vattr, names.vattr=names.vattr, use.subnet.cache=TRUE) %>% map(function(nw1){
     for(name in copy.ergmlhs) nw1%ergmlhs%name <- nw%ergmlhs%name
     nw1})
@@ -500,7 +537,7 @@ subnetwork_templates <- function(nw, split.vattr=".NetworkID", names.vattr=".Net
 .block_vertexmap <- function(nw, by=nw %n% ".blockID.vattr", same_dim=FALSE){
   a <- .peek_vattrv(nw, by)
   n <- length(a)
-  bip <- nw %n% "bipartite"
+  bip <- b1.size(nw)
   if(NVL(bip,0)){
     ea <- a[seq_len(bip)]
     aa <- a[bip+seq_len(n-bip)]
@@ -520,3 +557,18 @@ subnetwork_templates <- function(nw, split.vattr=".NetworkID", names.vattr=".Net
   }
   o
 }
+
+#' @noRd
+#' @importFrom networkLite as.networkLite
+#' @export
+as.networkLite.combined_networks <- function(x, ...){
+  if(!is.null(x %n% ".subnetcache")){
+    snc <- x %n% ".subnetcache"
+    snc[[1]] <- lapply(snc[[1]], as.networkLite)
+    x %n% ".subnetcache" <- snc
+  }
+  extra_classes <- class(x)[seq_len(which(class(x) == "combined_networks")[1])]
+  class(x) <- class(x)[-seq_len(which(class(x) == "combined_networks")[1])]
+  x <- as.networkLite(x)
+  structure(x, class = c(extra_classes, class(x)))
+}
