@@ -1,11 +1,11 @@
-#  File R/zzz.R in package ergm.multi, part of the
-#  Statnet suite of packages for network analysis, https://statnet.org .
+#  File R/zzz.R in package ergm.multi, part of the Statnet suite of packages
+#  for network analysis, https://statnet.org .
 #
-#  This software is distributed under the GPL-3 license.  It is free,
-#  open source, and has the attribution requirements (GPL Section 7) at
+#  This software is distributed under the GPL-3 license.  It is free, open
+#  source, and has the attribution requirements (GPL Section 7) at
 #  https://statnet.org/attribution .
 #
-#  Copyright 2003-2024 Statnet Commons
+#  Copyright 2003-2025 Statnet Commons
 ################################################################################
 #' @import ergm
 #' @import network
@@ -29,8 +29,8 @@
 
   eval(COLLATE_ALL_MY_CONTROLS_EXPR)
 
-  .RegisterProposals()
   .RegisterKeywords()
+  .RegisterCombiners()
 }
 
 ## BEGIN boilerplate: should be kept in sync with statnet.common.
@@ -54,15 +54,15 @@ snctrl <- statnet.common::snctrl
 eval(UPDATE_MY_SCTRL_EXPR)
 ## END boilerplate: should be kept in sync with statnet.common.
 
-.RegisterProposals <- function(){
-  ergm_proposal_table("c", "Bernoulli", "|bd|bdmax&blockdiag", 0, "random", "blockdiag")
-  ergm_proposal_table("c", "Bernoulli", "|bd|bdmax&blockdiag&sparse", 1, "TNT", "blockdiagTNT")
-}
-
 .RegisterKeywords <- function() {
   ergm_keyword(name="layer-aware", short="layer", description="operates on multilayer network constructs", popular=TRUE, package="ergm.multi")
 }
 
+.RegisterCombiners <- function() {
+  ergm.multi_combiner(".LayerID", c("Layer()", "layers"))
+  ergm.multi_combiner(".NetworkID", c("Networks()", "networks"))
+}
+
 #' @useDynLib ergm.multi
 .onUnload <- function(libpath){
   library.dynam.unload("ergm.multi",libpath)
