@@ -1,11 +1,11 @@
-/*  File src/changestats_multinet.c in package ergm.multi, part of the
- *  Statnet suite of packages for network analysis, https://statnet.org .
+/*  File src/changestats_multinet.c in package ergm.multi, part of the Statnet
+ *  suite of packages for network analysis, https://statnet.org .
  *
- *  This software is distributed under the GPL-3 license.  It is free,
- *  open source, and has the attribution requirements (GPL Section 7) at
+ *  This software is distributed under the GPL-3 license.  It is free, open
+ *  source, and has the attribution requirements (GPL Section 7) at
  *  https://statnet.org/attribution .
  *
- *  Copyright 2003-2024 Statnet Commons
+ *  Copyright 2003-2025 Statnet Commons
  */
 #include "ergm.multi_changestat_multinet.h"
 #include "ergm_changestat_operator.h"
@@ -38,7 +38,7 @@ I_CHANGESTAT_FN(i__subnets){
       lnnodes = *(iinputs++);
     }
 
-    sn->onwp[i] = NetworkInitialize(NULL, NULL, 0, lnnodes, DIRECTED, lbip, 0, 0, NULL);
+    sn->onwp[i] = NetworkInitialize(NULL, NULL, 0, lnnodes, DIRECTED, lbip);
   }
   
   EXEC_THROUGH_NET_EDGES_PRE(t, h, e, {
