@@ -1,11 +1,11 @@
 #  File tests/testthat/test-multilayer.R in package ergm.multi, part of the
 #  Statnet suite of packages for network analysis, https://statnet.org .
 #
-#  This software is distributed under the GPL-3 license.  It is free,
-#  open source, and has the attribution requirements (GPL Section 7) at
+#  This software is distributed under the GPL-3 license.  It is free, open
+#  source, and has the attribution requirements (GPL Section 7) at
 #  https://statnet.org/attribution .
 #
-#  Copyright 2003-2024 Statnet Commons
+#  Copyright 2003-2025 Statnet Commons
 ################################################################################
 # "Correct" transitivity calculators
 dediag <- function(m, x=0) {diag(m) <- x; m}
@@ -69,8 +69,8 @@ test_that("twostarL statistics for undirected networks", {
   lnw <- Layer(nw1, nw2)
 
   sim <- suppressWarnings(simulate(lnw~
-                    twostarL(c(~`1`,~`2`), "any",FALSE)+
-                    twostarL(c(~`1`,~`2`), "any", TRUE),
+                    twostarL(c(~`1`,~`2`), distinct = FALSE)+
+                    twostarL(c(~`1`,~`2`)),
                   control=ctrl,
                   nsim=200))
 
@@ -385,3 +385,8 @@ test_that("Statistics simulation for heterogeneously bipartite networks", {
   expect_equal(summary(slnw~edges+L(~edges,~`1`)+L(~edges,~`2`)+L(~edges,~(`1`|`2`))+L(~edges,~(`1`&`2`))),
                c(85,10,75,85,0), ignore_attr=TRUE)
 })
+
+test_that("L() stops if given a non-multilayer object with a sensible error message.",{
+  expect_error(ergm_model(nw1 ~ L(~edges)),
+               "In term 'L' in package 'ergm\\.multi': The LHS of the model is not a multilayer 'Layer\\(\\)' construct\\.")
+})
